#
# Qualitative analysis module for mced
#
# Module grouping the procedures for "qualitative" analysis
# - checking topological properties (not done yet!)
# - probability consistency
#
######################################################################
# Check_Property -
# General handling procedure for calling a boolean function and displaying
# a message according to the result. If the result is false, then it is
# assumed that there is a global variable containing the bad objects.
# It is then proposed to display (highlight) them.
#
# Parameters:
# - test_function	the boolean function to call
# - title	title of the popup window
# - description	a string describing the property, used in the message.
#
proc Check_Property { test_function title description } {
    global bad_obj_list

    set res [$test_function]
    if { $res == 1 } {
	set message "This is $description."
    } elseif { $res == "unknown" } {
	set message "Can't tell if it is $description."
    } elseif { $res == 0 } {
	set message "This is not $description."
    }

    if { $res == 0 } {
	switch [tk_dialog .graph.property "Bad Properties" "$message\nDo you wish to see the bad objects?" {} 0 {Yes} {No}] {
	    0 { Select_Object_List .graph.canvas $bad_obj_list }
	    1 { return }

	}
    } else {
	ReportInfo $title $message
    }
}

#######################################################################
# Check_Proba_Matrix -
# Front-end procedures to the corresponding boolean procedures
#
# Parameters: none
#
proc Check_Proba_Matrix { } {

    Check_Property Is_Proba_Matrix "Probability Matrix Analysis" \
	"a Probability Matrix" 

}

######################################################################
# Is_Proba_Matrix -
# Boolean function returning true if the graph has a stochastic matrix,
# that is, that all probas are in [0,1] AND sum up to one.
#
# Parameters: none
#
proc Is_Proba_Matrix { } {
    global node_list arc_list arc_proba bad_obj_list

    set bad_obj_list {}
    set ok 1

    # initializing proba to 0
    foreach node $node_list {
	set total([lindex $node 0]) 0
    }

    # accumulating proba
    foreach arc $arc_list {
	set node_id [lindex $arc 1]
	set proba $arc_proba([lindex $arc 0])
	if [isReal $proba] {
	    if [expr { $proba < 0.0 } || { $proba > 1.0 } ] {
		set ok 0
	    }
	} else {
	    set ok 0
	}
	if $ok {
	    set total($node_id) [expr $total($node_id)+$proba]
	} else {
	    lappend bad_obj_list [lindex $arc 0]
	}
    }

    # checking that proba is 1
    foreach node $node_list {
	set node_id [lindex $node 0]
	if [expr $total($node_id) != 1.0] {
	    set ok 0
	    lappend bad_obj_list $node_id
	}
    }
    
    return $ok
}

######################################################################
# Is_Empty_Graph -
# A boolean function answering true if the current Petri Net has no
# objects at all.
#
# Parameters: none
#
proc Is_Empty_Graph { } {
    global node_list arc_list

    return [expr [llength $node_list]+[llength $arc_list] == 0 ]
}

