#
# The module for processing the options of the executables, and preparing
# the help text that will be displayed in the application windows.
#
######################################################################
# ConfigureButton -
# Procedure to build the frame with application buttons from the description
# read in the file .Mtoolrc. Displays the buttons of programs and load 
# reference files for help.
#
# Parameters:
# - original	forces the rc file to be the one in the distribution, not
#		the user's one.
#
proc  ConfigureButton { original } {
    global man whence rcfile
    global cmd_line env where_is_Mtool
    
    set orig_rcfile "$where_is_Mtool/$rcfile"

    if $original {
	# I hope it's there!
	set Id_config [open $orig_rcfile r]
    } elseif { [file exists "./$rcfile"] } {
	puts -nonewline "./$rcfile found, trying to use it ... "
	flush stdout
	if {[catch {open "./$rcfile" r} Id_config] != 0} {
	    puts -nonewline "oups\nCan not open it !\nTrying to open $orig_rcfile  ... " 
	    flush stdout
	    if { [catch {open "$newrcfile" r} Id_config] != 0} {
		puts "oups\nCan not open it ! BYE"
		exit
	    } else {
		puts "ok"
	    }
	} else {
	    puts "ok"
	}
    } elseif [file exists "~/$rcfile"] {
	puts -nonewline "~/$rcfile found, trying to use it ... "
	flush stdout
	if { [catch {open "~/$rcfile" r} Id_config] != 0} {
	    puts -nonewline "oups\nCan not open it !\nTrying to open $orig_rcfile  ... " 
	    flush stdout
	    if { [catch {open "$orig_rcfile" r} Id_config] != 0} {
		puts "oups\nCan not open it ! BYE"
		exit
	    } else {
		puts "ok"
	    }
	} else {
	    puts "ok"
	}
    } else {
	puts -nonewline "First use of mtool.\nCopying $rcfile in home directory ... "
	flush stdout
	if { [catch "exec cp $newrcfile $env(HOME)"] == 0 } {
	    puts "ok"
	    flush stdout
	    if { [catch {open "~/$rcfile" r} Id_config] !=0 } {
		puts "$rcfile copied in ${env(HOME)} but can not open it"
		exit
	    }
	} else {
	    puts "oups\nCan not copy ! BYE"
	    exit
	}
    }

    # Here, the Id_config file should have been chosen...

    set group_nb 0
    set S 0

    while { [gets $Id_config line]!=-1 } {
	if { [string length $line] > 0 && [string compare [string index $line 0] "\#"] } {
	    if { [string compare [string range $line 0 4] "group"] == 0 } {
		# look for group name, if any
		set grpname [string range $line 5 end]
		set S 0
		incr group_nb
		frame .frame.group$group_nb
		label .frame.group$group_nb.label -text $grpname
		pack .frame.group$group_nb.label -side top
		pack .frame.group$group_nb -fill x -fill y -side left
		puts "\nNew group (\#$group_nb) name: $grpname"
	    } elseif { [string compare $line "reference"] == 0 } {
 		gets $Id_config r
		# Not loading reference anymore, due to a problem in LoadRef
# 		puts -nonewline "\nLoading reference \t... "
# 		flush stdout
# 		LoadRef $r
# 		puts "ok"
	    } elseif { [string compare $line "standard"] == 0 } {
		puts "\nStandard group"
		set S 1
	    } elseif { !$S } {
		set name [file tail [lindex $line 0]]
		set win_name [ToLower $name]
		set whence($name) [lindex $line 0]
		set man($name) [lindex $line 1]
		if  { [FindOptions $name] } {
		    set cmd_line($name) ""
		    button .frame.group$group_nb.$win_name -text $name \
			-command "DisplayOptions $name"
		    pack .frame.group$group_nb.$win_name -fill x \
			-side top -padx 2m -pady 1m
		}
	    } else {
		set name [file tail [lindex $line 0]]
		puts "$name"
		set win_name [ToLower $name]
		set whence($name) [lindex $line 0]
		button .botbin.$win_name -text $name -relief ridge -bd 3 \
		    -command "StandBin $name $win_name"
		pack .botbin.$win_name -side left -padx 2m -pady 1
	    }
	}
    }
    close $Id_config
}

######################################################################
# StandBin -
# Execute an application from the "standard" group, that is, the one
# for which no options are known, and the name of the file is not
# preceded with "-f".
#
# Parameters:
# - name	name of the application
# - win_name	name of the window for the popup in case of errors
#
proc  StandBin { name win_name } {
    global in_file
    global whence

    catch {.frame.list get [.frame.list cursingle]} file
    if { [file isfile $file] && [string compare $file ""] } {
	set in_file [pwd]
	append in_file "/"
	append in_file $file
#	set in_file $file
    } else {
	set in_file "SELECT A FILE !"
	bell
    }
    if { [string compare $in_file "SELECT A FILE !"] != 0 && [string compare $in_file ""] }  {
	puts "Executing $whence($name) $in_file"
	exec $whence($name) $in_file &
    } else {
	tk_dialog .$win_name "$name error" "No input file selected" {error} 0 OK
    }
}

######################################################################
# FindOptions -
# Function to get the list of options recognized by an application.
# Executes a certain program `name_prg` with the `-h` option.
# Returns 1 if the program exited normally, 0 otherwise.
#
# Parameters:
# - name_prg	name of the program to be executed
#
proc  FindOptions { name_prg } {
    global whence options man env 
    
    set not_find [catch {exec $whence($name_prg) -h} usage]
    puts -nonewline "$name_prg   \t..."
    flush stdout

    if $not_find {
	puts " err"
	puts $usage
	return 0
    } else {
	set list_options [SplitOptions $usage]
	set options($name_prg) [CutHelpOptions $name_prg $list_options]
	InitHelpOptions $name_prg
	if { "$man($name_prg)" == "" } {
	    puts " ok but no help file specified"
	} elseif ![file exists $man($name_prg)] {
	    # perhaps it is relative to ERSHOME?
	    if [info exists env(ERSHOME)] {
		set fullname "$env(ERSHOME)/$man($name_prg)"
		if ![file exists $fullname] {
		    puts " ok but can't find help file"
		} else {
		    set man($name_prg) $fullname
		    puts " ok"
		}
	    } else {
		puts " ok but can't find help file"
	    }
	} else {
	    puts " ok"
	}
	return 1
    }
}

######################################################################
# InitHelpOptions -
# Initialize the on-line help of each option with the default string
# "No help on this option".
#
# Parameters:
# - name_prg	name of the program, the help of which is initialized
#
proc InitHelpOptions { name_prg } {
    global options help_opt
    
    foreach i $options($name_prg) {
	regsub {^ *- *} $i {} opts
	set opt [lindex $opts 0]	
	set help_opt($name_prg,$opt) "No help on this option"
    }
}

######################################################################
# SplitOptions -
# Function return the options of an application as a list.
# It removes the brackets from the initial options list.
# 
# Parameters:
# - usage	the output of execution of 'prg_name -h'
#
proc  SplitOptions {usage} {
    set max [string length $usage]
    
    for {set i 0} {$i < $max} {incr i} {
	set char [string index $usage $i]
	if { ![string compare "\[" $char] } {
	    set first [expr $i +1]
	} elseif { ![string compare "\]" $char] } {
	    lappend list_options [string range $usage $first [expr $i -1]]
	}
    }
#    puts "Options found: $list_options"
    return $list_options
}

######################################################################
# CutHelpOptions -
# A function to suppress options like "-h" "-?" from the list of options.
# Also suppresses "-f" but after configuring the has_input variable.
#
# Parameters:
# - name_prg	the name of the application being processed
# - list_options	the original list of all options
#
proc  CutHelpOptions {name_prg list_options} {
    global has_input

    set has_input($name_prg) 0

    foreach i {{-h$} {--help$} {-\?$} {-help$} {-f }} {
	set num [lsearch -regexp $list_options "$i"]
	if { $num != -1 } {
	    if { $i == {-f } } {
		set has_input($name_prg) 1
	    }
	    set list_options [lreplace $list_options $num $num]
	}
    }

    if $has_input($name_prg) {
	puts -nonewline " <$name_prg has input>"
    } else {
	puts -nonewline " <$name_prg has no input>"
    }

    return $list_options
}

######################################################################
# HelpOnEveryOptions -
# Takes as input the name of a program in the list (in .Mtoolrc )
# and finds the on-line help for each options.
# It puts the result in the global variables man() and help_opt()
#
# Parameters:
# - name_prg	the name of the program to be analyzed
#
proc  HelpOnEveryOptions {name_prg} {
    global options man help_opt
    
    set Id_man [open $man($name_prg) r]
    set in_option 0
    while { [gets $Id_man line]!=-1 } {
	if [regexp {^ *\\option\{ *-} $line] {
	    if { $in_option } {
		set help_opt($name_prg,$one_option) [TakeEndOption $tmp]
	    } else {
		set in_option 1
	    }
	    regsub {^ *\\option\{ *- *} $line {} opt
	    regsub {\}} [lindex $opt 0] {} one_option
	    set tmp $line
	} elseif { [regexp {^ *\} *$} $line] && $in_option } {
	    set help_opt($name_prg,$one_option) [TakeEndOption $tmp]
	    break
	} elseif { $in_option } {	
	    append tmp " $line"
	}
    }
    close $Id_man
}

######################################################################
# TakeEntOption -
# Function returning the string after the '}' character in some sting.
# Used to keep the end of option in line such as
# "\option{-debug} string_to_keep".
#
# Parameters:
# - line	the original string
#
proc  TakeEndOption { line } {
    set first [string first "\}" "$line"]
    incr first
    set end_line [string range $line $first end]
    # Ajout de Detex au cas ou ...
    set end_line [TranslateText $end_line]
    if [ string match {*\\*} "$end_line" ] {
	set end_line [Detex $end_line]
    }
    return $end_line
}

######################################################################
# DefaultOnEveryOption -
# Procedure to find the default expression for all options of some
# application.
#
# Parameters:
# - name_prg	name of the program
#
proc  DefaultOnEveryOptions { name_prg } {
    global help_opt options type_opt type_name default_opt
    
    set copy_options $options($name_prg)
    set help_opt($name_prg,maximum) 55
    foreach i $options($name_prg) {
	regsub {^ *- *} $i {} opts
	set opt [lindex $opts 0]
	if { [llength $i] == 2 } {
	    set param [lindex $i 1]
	    switch -regexp $param {
		number -
		level -
		size   -
		int -
		time { 
		    set type_opt($name_prg,$opt) number
		    set type_name($name_prg,$opt) "integer"
		    if [regexp -nocase {[;,:] *([0-9]+) *[a-z ]*default} \
			    $help_opt($name_prg,$opt) lost default_opt($name_prg,$opt)] {
			regsub $lost $help_opt($name_prg,$opt) \
			    {} help_opt($name_prg,$opt)
		    } else {
			set default_opt($name_prg,$opt) ""
		    }
		}
		reel -
		double -
		float -
		real { 
		    set type_opt($name_prg,$opt) real
		    set type_name($name_prg,$opt) "real"
		    if [regexp -nocase {[;,:] *([0-9]+) *[a-z ]*default} \
			    $help_opt($name_prg,$opt) lost default_opt($name_prg,$opt)] {
			regsub $lost $help_opt($name_prg,$opt) \
			    {} help_opt($name_prg,$opt)
		    } else {
			set default_opt($name_prg,$opt) ""
		    }
		}		
		file { 
		    set type_opt($name_prg,$opt) file
		    set type_name($name_prg,$opt) "file name"
		    set default_opt($name_prg,$opt) "output"
		}
		
		dir {
		    set type_opt($name_prg,$opt) dir
		    set type_name($name_prg,$opt) "directory name"
		    if [regexp -nocase {[;,:] *[a-z ]*default[a-z ]*current[a-z ]*direct[a-z ]*} $help_opt($name_prg,$opt) lost] {
			regsub $lost $help_opt($name_prg,$opt) {} help_opt($name_prg,$opt)
			set default_opt($name_prg,$opt) "."
		    } else {
			set default_opt($name_prg,$opt) ""
		    }
		}

		default {
		    set type_opt($name_prg,$opt) unknown
		    set type_name($name_prg,$opt) "thing"
		    if [regexp -nocase {[;,:] *([0-9]+) *[a-z ]*default} \
			    $help_opt($name_prg,$opt) lost default_opt($name_prg,$opt)] {
			regsub $lost $help_opt($name_prg,$opt) \
			    {} help_opt($name_prg,$opt)
		    } elseif [regexp -nocase {[;,:] *[a-z ]*default[a-z ]*current[a-z ]*direct[a-z ]*} $help_opt($name_prg,$opt) lost] {
			regsub $lost $help_opt($name_prg,$opt) {} help_opt($name_prg,$opt)
			set default_opt($name_prg,$opt) "."
		    } else {
			set default_opt($name_prg,$opt) ""
		    }
		}
	    }
	} elseif { [regexp -nocase {[;:,] *this[a-z0-9 ]*default} $help_opt($name_prg,$opt) ] } {
	    set num [lsearch $copy_options $i]
	    if { $num!=-1 } {
		set copy_options [lreplace $copy_options $num $num]
	    }
	} else {
	    set default_opt($name_prg,$opt) ""
	    set type_opt($name_prg,$opt) flag
	    set type_name($name_prg,$opt) "switch"
	}

#	puts "Type of $name_prg/$opt set to \"$type_opt($name_prg,$opt)\""
	
	set max [string length $help_opt($name_prg,$opt)]
	if { $max <= 100 && $max > $help_opt($name_prg,maximum) } {
	    set help_opt($name_prg,maximum) $max
	}
	# garde pour limiter la largeur des fenetres
    }
    set options($name_prg) $copy_options
}

######################################################################
# DisplayOptions -
# Display options of an application with on-line help (if this option
# activated).
#
# Parameters:
# - name	the name of the application
#
proc  DisplayOptions { name } {
    global options labelFont help_opt has_input
    global in_file cmd_line man
    global DEBUG VERBOSE

    if { "$man($name)" != "" && [file exists $man($name)] } {
	if { ![info exists help_opt($name,h)] } {
	    HelpOnEveryOptions $name
	    DefaultOnEveryOptions $name
	}
	set win ".[ToLower $name]"
	if [catch {toplevel $win}] { return }
	wm title $win "Options $name"
	set name_icon "Option $name"
	wm iconname $win $name_icon
	# Uncomment the line below to iconifie child with parent
	wm transient $win .
#	set geom_parent [split [wm geometry .] x+]
#	set geom "+[expr [lindex $geom_parent 0]*7+135+[lindex $geom_parent 2]]+[lindex $geom_parent 3]"
#	wm geometry $win $geom
	
	frame $win.upside -relief raised -bd 1
	pack $win.upside -fill x
	
	button $win.upside.return -relief raised -borderwidth 2 -text "EXIT" \
	    -cursor pirate -command "destroy $win"
	if $VERBOSE {
	    bind $win.upside.return <Enter> "DisplayInfo option_info($name) \"Close the options window of $name\"" 
	    bind $win.upside.return <Leave> "DisplayInfo option_info($name) \"\""
	}
	
	label $win.upside.name -text "$name" \
	    -font "-adobe-new century schoolbook-*-i-*-*-18-*-*-*-*-*-*-*"
	button $win.upside.help -relief raised -borderwidth 2 -text "HELP" \
	    -command "EditHelp $name"
	if $VERBOSE {
#	    bind $win.upside.help <Enter> "tk_butEnter %W ; DisplayInfo option_info($name) \"Affiche l'aide concernant la commande $name\""	
	    bind $win.upside.help <Enter> "DisplayInfo option_info($name) \"Display help about command $name\""	
#	    bind $win.upside.help <Leave> "tk_butLeave %W ; DisplayInfo option_info($name) \"\""
            bind $win.upside.help <Leave> "DisplayInfo option_info($name) \"\""
	}
	pack $win.upside.return $win.upside.name $win.upside.help -side left \
	    -expand yes -fill x

	frame $win.file -relief flat
	pack $win.file -ipady 3 -fill x
	
	button $win.file.run -text "run" \
	    -font "-adobe-new century schoolbook-*-i-*-*-18-*-*-*-*-*-*-*" \
	    -command "Execute $name"
	if $VERBOSE {
	    bind $win.file.run <Enter> "DisplayInfo option_info($name) \"Execute command $name with specified options\""
	    bind $win.file.run <Leave> "DisplayInfo option_info($name) \"\""
	}
	pack $win.file.run -side right -padx 3 -fill y
	
	label $win.file.in -textvariable in_file -borderwidth 2 \
	    -font 8x13bold
	pack $win.file.in -pady 2 -expand yes
	
	#   for each options
	#   i build the checkbutton
	frame $win.checkbutton 
	pack $win.checkbutton -fill x

	# Make a first pass in order to cleanup option names, estimate total
	# width of button row and split it if necessary.
	set cur_char_nb 0
	set opt_list {}
	foreach i $options($name) {
	    regsub {^ *- *} $i {} opts
	    set opt [lindex $opts 0]
	    lappend opt_list $opt
	    # add width of text, plus 2 characters of "overhead"
	    incr cur_char_nb [string length $opt]
	    incr cur_char_nb 2
	}
	set nbrows [expr 1 + ($cur_char_nb / 60) ]
	set nbcols [expr [llength $opt_list] / $nbrows]
	if $DEBUG {
	    puts "Have to make $nbrows rows"
	}

	for { set i 0 } { $i < $nbrows } { incr i 1 } {
	    set opt_row($i) [lrange $opt_list \
				 [expr $i*$nbcols] \
				 [expr (1+$i)*$nbcols] ]
	    if $DEBUG {
		puts "Row $i: $opt_row($i)"
	    }
	}

	for { set i 0 } { $i < $nbrows } { incr i 1 } {

	    set f [frame $win.checkbutton.row$i]

	    foreach opt $opt_row($i) {
	    
		set opt_win [ToLower $opt]
	    
		checkbutton $f.$opt_win -text "$opt " \
		    -relief raised -bd 1 \
		    -variable "${name}_${opt}_check" \
		    -command "AckOption $win $name $opt"
	    
		pack $win.checkbutton.row$i.$opt_win \
		    -side left -expand yes -fill x

		if $VERBOSE {
		    bind $f.$opt_win <Enter> "
		DisplayInfo option_info($name) \"$help_opt($name,$opt)\" "
		    bind $f.$opt_win <Leave> "
		DisplayInfo option_info($name) \"\" "
		}
	    }

	    pack $f -fill x
	}
	
	frame $win.center -relief ridge -bd 2
	pack $win.center -fill x
	
	label $win.center.label -textvariable ${name}_center_label -height 3
	pack $win.center.label -fill x
	
	if $VERBOSE {
	    label $win.info -width [expr $help_opt($name,maximum)-5] \
		-wraplength 600 -height 2 \
		-textvariable option_info($name)
	    pack $win.info
	}

	catch {.frame.list cursingle} val
	catch {.frame.list get [.frame.list cursingle]} file
#	puts "<$val|$file>"
	if !$has_input($name) {
	    set in_file ""
	} elseif { [string compare $val ""] && \
	     [file isfile $file] && [string compare $file ""] } {
	    set in_file $file
	} else {
	    set in_file "SELECT A FILE !"
	    bell
	}
	
    } else {
	tk_dialog .error "$name" "The specified help file does not exist" {warning} 0 "OOPS"
    }
}

######################################################################
# AckOption -
# A procedure to manage the state of each options (selected or not),
# add selected option in command line and suppress those which are unselected.
#
# Parameters:
# - win		window in wich options are diplayed
# - name	name of program
# - opt		the option currently selected
#
proc  AckOption { win name opt } {
    global type_opt type_name default_opt
    global cmd_line

    uplevel "
        if { $${name}_${opt}_check && [string compare $type_opt($name,$opt) flag]} {
	    $win.center.label config -height 2
	   set ${name}_center_label \"Option $opt\"
	   label $win.center.type -text \"Enter a $type_name($name,$opt)\"
	   pack $win.center.type -side left
	   emacsEntry $win.center.entry -textvariable ${name}_${opt}_entry -relief sunken -bd 2
	   pack $win.center.entry
	   set ${name}_${opt}_entry $default_opt($name,$opt)
	   after 100
	   \# grab set $win.center.entry
	   focus $win.center.entry
	   bind $win.center.entry <Return> {
               if \[CheckEntry $name $opt $${name}_${opt}_entry\] {
  	        set default_opt($name,$opt) $${name}_${opt}_entry
#	        grab release $win.center.entry
#	        focus none
	        destroy $win.center.type
	        destroy $win.center.entry
                lappend cmd_line($name) \"-$opt $${name}_${opt}_entry\"
                $win.center.label config -height 3
                set ${name}_center_label \"Slection : \$cmd_line($name)\"
               }
	   }
           bind $win.center.entry <Escape> {
	       grab release $win.center.entry
#	       focus none
	       destroy $win.center.type
	       destroy $win.center.entry
               $win.center.label config -height 3
               set ${name}_${opt}_check 0
               set length \[llength \$cmd_line($name)\]
               if { \$length==0 } {
                   set ${name}_center_label \"No selection\"
               } else {
                   set ${name}_center_label \"Selection : \$cmd_line($name)\"
               }
           }
	} elseif { $${name}_${opt}_check } {
           set ${name}_center_label \"Option $opt activated\"
           update
           after 1000
           lappend cmd_line($name) \"-$opt\"
           set ${name}_center_label \"Slection : \$cmd_line($name)\"
        } elseif !{[string compare $type_opt($name,$opt) flag]} {
           if {[set indice [lsearch -regexp $cmd_line($name) $opt]]!=-1 } {
               set cmd_line($name) \[lreplace \"\$cmd_line($name)\" $indice $indice\]
               set length \[llength \"$cmd_line($name)\"\]
               if { \$length==1 } {
                   set ${name}_center_label \"Aucune slection\"
               } else {
                   set ${name}_center_label \"Slection : \$cmd_line($name)\"
               }
           }
        } else {
           if {[set indice [lsearch -regexp $cmd_line($name) "$opt "]]!=-1 } {
               set cmd_line($name) \[lreplace \"\$cmd_line($name)\" $indice $indice\]
               set length \[llength \"$cmd_line($name)\"\]
               if { \$length==1 } {
                   set ${name}_center_label \"No selection\"
               } else {
                   set ${name}_center_label \"Selection : \$cmd_line($name)\"
               }
           }
       }
     "
}

######################################################################
# Execute -
# Procedure to execute a program with the currently selected options
# (contained in the global variable cmd_line) and display its output in a 
# text window.
#
# Parameters:
# - name	the name of the program to execute
#
proc  Execute { name } {
    global whence cmd_line in_file has_input

    set win_name ".[ToLower $name]"

    if { $has_input($name) \
	     && ![string compare $in_file "SELECT A FILE !"] } {
	tk_dialog ${win_name}.error "$name error" \
	    "No file selected" {error} 0 OK
	return
    }

    # prepare command line
    set cmd "$whence($name) "
    if $has_input($name) {
	append cmd "-f $in_file "
    }
    append cmd [join $cmd_line($name) { }]

    # prepare output window and execute
    if ![catch "eval exec $cmd" output] {

	# The old way to do it has been replaced (as of ERS 0.1.15) by the
	# new Logger. The old code is kept below in the dummy OldResultLogger
	# procedure in case of.

	# I don't know why, but a newline seems to be necessary with output.
	LogResult "Result of $cmd" "$output\n"

    } else {
	# The case of an error should be handled with a logger as well,
	# but I have not made my mind up: should it be the same logger,
	# or a different one?
	# In the meantime, I use a separate dialog window.
	set res [ tk_dialog ${win_name}.error "$name error" "Error executing $name" {error} 0 "OK" "See the errors" ]
	if { $res == 1 } {
	    #		puts "Ok. Je vais construire ${win_name}.showerror"
	    set top "${win_name}.showerror"
	    if ![catch {toplevel $top}] {
		#		    puts "Ok. Je construis $top"
		wm title $top "Errors of $name"
		set name_icon "Errors of $name"
		wm iconname $top $name_icon
		set geom_parent [split [wm geometry $win_name] x+]
		set geom "+[expr [lindex $geom_parent 1]+[lindex $geom_parent 3]+15]+[lindex $geom_parent 2]"
		wm geometry $top $geom
		
		button $top.ok -text CLOSE -command "destroy $top"
		button $top.save -text SAVE -command "Save $top.t"
		EmacsText $top.t -relief raised -bd 2 \
		    -yscrollcommand "$top.s set" \
		    -width 60 -height 30 -wrap word -setgrid yes
		scrollbar $top.s -relief flat -command "$top.t yview"
		pack $top.s -side right -fill y
		pack $top.t -expand yes -fill both
		pack $top.ok $top.save \
		    -side left -fill x -padx 2
		$top.t insert insert "$output"
	    }
	}
    }
}

######################################################################
# CheckEntry -
# Boolean function returning true if some entry (a string) has a specified
# type.
#
# Parameters:
# - name	name of the application in question
# - opt		option to test
# - value	string to check
#
proc CheckEntry {name opt value} {
    global type_opt

#    puts "CheckEntry /$name/$opt/$value"

    switch $type_opt($name,$opt) {
	number {
	    if ![isNumber $value] {
		DisplayInfo option_info($name) \
		    "$opt must be an integer !! $value is not" 
		return 0
	    } else {
		return 1
	    }
	}
	real {
	    if ![isReal $value] {
		DisplayInfo option_info($name) "$opt must be a real !!" 
		return 0
	    } else {
		return 1
	    }
	}
	file {
	    if [file exists $value] {
		return [tk_dialog .file_exist "$opt option of $name" "File $value already exists" {warning} 1 Cancel Continue]
	    }
	    return 1
	}
	dir {
	    if [file isdirectory $value] {
		return 1
	    } elseif [file exists $value] {
		tk_dialog .file_exist "$opt option of $name" "$value is not a directory" {warning} 0 ok
		return 1
	    } else {
		tk_dialog .file_exist "$opt option of $name" "$value does not exist" {warning} 0 ok
		return 1
	    }
	}
	unknown {
	    return 1
	}
    }
}

######################################################################
# OldResultLogger -
# Below is the old code block for displaying results. It is kept here
# because in includes some features that are not reproduced in the new
# Logger: EmacsText, geometry placement of the window, etc.
#
proc OldResultLogger { } {

    set top "${win_name}.output"

	if ![catch {toplevel $top}] {
	    wm title $top "Output $name"
	    set name_icon "Output $name"
	    wm iconname $top $name_icon
	    # Permet d'iconifier le fils a partir du pere
	    #    wm transient $win .
	    set geom_parent [split [wm geometry $win_name] x+]
	    set geom "+[expr [lindex $geom_parent 0]+[lindex $geom_parent 2]+15]+[lindex $geom_parent 3]"
	    wm geometry $top $geom
	    
	    button $top.ok -text CLOSE -command "destroy $top"
	    button $top.refresh -text CLEAR \
		-command "$top.t delete 0.0 end"
	    button $top.save -text SAVE -command "Save $top.t"
	    EmacsText $top.t -relief raised -bd 2 \
		-yscrollcommand "$top.s set" \
		-width 60 -height 30 -wrap word -setgrid yes
	    scrollbar $top.s -relief flat -command "$top.t yview"
	    pack $top.s -side right -fill y
	    pack $top.t -expand yes -fill both
	    pack $top.ok $top.refresh $top.save -side left -fill x -padx 2
	}
	$top.t insert insert "$output\n\n"
}