#
# Module grouping the procedures for creating new graphs
#
######################################################################
# Control_Panel -
# Creates a user dialog interface for specifying the properties of
# the created graph: attributes of places/transitions/links and geometry.
#
# Parameters:
# - win		the canvas widget where the graph is created
#
proc Control_Panel { win } {
    global create_props TransLawParams PlaceLawParams

    frame $win.trans
    label $win.trans.label  -text "Transition Attributes" -width 20 -anchor w
    label $win.trans.tlabel -text "Time" 

    # create law browsing area, setting the parameters to that of 
    # the create properties
    array set TransLawParams $create_props(TransTime)
    set tlb [Law_Browser $win.trans TransLawParams]

    label $win.trans.rlabel -text "Recycling" 
    radiobutton $win.trans.recycl -text "Yes" \
	-variable create_props(is_recycled) \
	-value yes
    radiobutton $win.trans.norecycl -text "No" \
	-variable create_props(is_recycled) \
	-value no
    pack $win.trans.label $win.trans.tlabel -pady 2 -side left -anchor n
    pack $tlb $win.trans.rlabel $win.trans.recycl $win.trans.norecycl \
	-side left
    pack $win.trans -side top -fill x

    frame $win.place
    label $win.place.label  -text "Place Attributes" -width 20 -anchor w
    label $win.place.tlabel -text "Time"

    # create law browsing area, setting the parameters to that of 
    # the create properties
    array set PlaceLawParams $create_props(PlaceTime)
    set plb [Law_Browser $win.place PlaceLawParams]

    label $win.place.mlabel -text "Marking" 
    entry $win.place.mark   -textvar create_props(place_marking) \
	-width 4 -relief sunken
    pack $win.place.label $win.place.tlabel -pady 2 -side left -anchor n
    pack $plb $win.place.mlabel $win.place.mark -side left
    pack $win.place -side top -fill x

    frame $win.link
    label $win.link.label  -text "Link Attributes" -width 20 -anchor w
    label $win.link.tlabel -text "Weight"
    entry $win.link.weight -textvar create_props(link_weight) \
	-width 4 -relief sunken
    pack $win.link.label $win.link.tlabel $win.link.weight -side left
    pack $win.link -side top -fill x
        
    frame $win.offset
    label $win.offset.label -text "Offset (mm)" -width 20 -anchor w
    label $win.offset.xlabel -text "X=" -width 3 -anchor w
    entry $win.offset.xoffset -textvar create_props(xoffset) \
	-width 4 -relief sunken
    label $win.offset.ylabel -text "Y=" -width 3 -anchor w
    entry $win.offset.yoffset -textvar create_props(yoffset) \
	-width 4 -relief sunken
    pack $win.offset.label $win.offset.xlabel $win.offset.xoffset \
	$win.offset.ylabel $win.offset.yoffset -side left 
    pack $win.offset -side top -fill x

    return [list $win.trans $win.place]
}

######################################################################
# SetCreateLaw -
# Procedure to put into the global creation properties the temporary
# arrays filled through the law browser.
#
# Parameters: none
#
proc SetCreateLaws { } {
    global create_props TransLawParams PlaceLawParams

    # puts "UPD with [array get TransLawParams]"
    set create_props(TransTime) "[array get TransLawParams]"
    set create_props(PlaceTime) "[array get PlaceLawParams]"
}

proc Create_Place_With_Props { win xpos ypos } {
    global create_props

    if [info exists create_props(PlaceTime)] {
	set plopt "-lawparams [list $create_props(PlaceTime)]"
    } else {
	set plopt ""
    }
    if [info exists create_props(place_marking)] {
	set mkopt "-marking $create_props(place_marking)"
    } else {
	set mkopt ""
    }
	
    set pl_id [Make_Place $win "-x $xpos -y $ypos $plopt $mkopt" ]

    return $pl_id
}

proc Create_Trans_With_Props { win xpos ypos } {
    global create_props

    if [info exists create_props(TransTime)] {
	set paopt "-lawparams [list $create_props(TransTime)]"
    } else {
	set paopt ""
    }
	
    set tr_id [Make_Trans $win "-x $xpos -y $ypos $paopt" ]

    return $tr_id
}


proc Some_Diags { size chordlist } {

    set found 0

    for { set i 0 } { [expr (!$found) && ($i<[llength $chordlist])] } \
	{incr i 1} {
	    set compl [expr $size-[lindex $chordlist $i]]
	    set found [expr -1 !=[lsearch -exact $chordlist $compl]]
    }

    return $found
}

proc Create_Cycle { win size chordlist } {
    global mark link_weight
    global PI curzoomvalue object_distance place_radius create_props
    global DEBUG

    if { $size <= 0 } { return }

    set scale [expr $curzoomvalue/100.0]
    set OD [expr $object_distance*$scale]

    set Alpha [expr 2*$PI / $size];
    if [ expr { sin($Alpha/2.0) == 0.0 } ] {
	set Radius 0.0
    } else {
	set Radius [expr $OD / sin( $Alpha/2 )]
    }

    # putting the center... at the center
    set val $create_props(xoffset)
    set xcenter [expr [winfo pixels . ${val}m]+$OD+$Radius]
    set val $create_props(yoffset)
    set ycenter [expr [winfo pixels . ${val}m]+$OD+$Radius]

    # perhaps it is necessary to resize the canvas?
    Resize_Graph $win [expr $xcenter+$Radius+$OD] \
	[expr $ycenter+$Radius+$OD]

    if $DEBUG {
	puts -nonewline "Creating cycle with $size trans. and chords $chordlist..."
    }

    # creation of the nodes 
    for { set i 0 } { $i < $size } { incr i 1 } {
	set xpos [xcoord $xcenter $Radius [expr $i*$Alpha]]
	set ypos [ycoord $ycenter $Radius [expr $i*$Alpha]]

	set tr_id($i) [Create_Trans_With_Props $win $xpos $ypos]

	if { "yes" == $create_props(is_recycled) } {
	    Create_Recycling $win $tr_id($i)
	}
    }

    # creation of the chords
    if $DEBUG {
	puts "[llength $chordlist] chords.."
    }
    foreach chord_length $chordlist {
	if { $chord_length < $size } {
	    if $DEBUG {
		puts "doing chord $chord_length"
	    }

	    for { set i 0 } { $i < $size } { incr i 1 } {

		set xpos1 [xcoord $xcenter $Radius [expr $i*$Alpha]]
		set ypos1 [ycoord $ycenter $Radius [expr $i*$Alpha]]

		set k [expr ($i+$chord_length) % $size]
		set xpos2 [xcoord $xcenter $Radius [expr $k*$Alpha]]
		set ypos2 [ycoord $ycenter $Radius [expr $k*$Alpha]]

		# put a place in between
		if [expr $chord_length == 1] {
		    set xpos [xcoord $xcenter $Radius [expr ($i+0.5)*$Alpha]]
		    set ypos [ycoord $ycenter $Radius [expr ($i+0.5)*$Alpha]]
		} else {
		    set xpos [expr ($xpos1+$xpos2)/2]
		    set ypos [expr ($ypos1+$ypos2)/2]
		}

		set pl_id [Create_Place_With_Props $win $xpos $ypos]

		# draw links 
		set li1_id [Put_TraPla_Link $win $tr_id($i) $pl_id {}]
		set li2_id [Put_PlaTra_Link $win $pl_id $tr_id($k) {}]
		if [info exists create_props(link_weight)] {
		    set link_weight($li1_id) $create_props(link_weight)
		    set link_weight($li2_id) $create_props(link_weight)
		}

		# (re)write text. Necessary because information may have 
		# changed
		Display_Weight $win $li1_id
		Display_Weight $win $li2_id

	    }
	}
    }

    # check for superposed chords
    if [Some_Diags $size $chordlist] {
	ReportWarning "Create Warning" "Warning: some places are superposed."
    }
  
    if $DEBUG {
	puts " ...Done"
    }

}

proc Make_Cycle { win } {
    global size

    if [catch {toplevel .params}] { return }

    wm title .params "Cycle Parameters"

    frame .params.size
    label .params.size.label -text "Number of Nodes:" -width 15  -anchor w
    entry .params.size.field -textvar size -width 3 -relief sunken

    frame .params.chord
    label .params.chord.label -text "List of chords:" -width 15 -anchor w
    entry .params.chord.field -textvar chordlist -width 10 -relief sunken

    pack .params.size.label .params.size.field -side left
    pack .params.chord.label .params.chord.field -side left
    pack .params.size .params.chord -side top -fill x

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n SetCreateLaws\n Create_Cycle $win \$size \$chordlist"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.actions -side top

    bind .params <Escape> {.params.actions.cancel invoke}

}

proc Make_Complete { win } {

    global size

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Number of Nodes:"
    entry .params.size.field -textvar size -width 3 -relief sunken

    pack .params.size.label .params.size.field -side left
    pack .params.size -side top -fill x

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n SetCreateLaws\n Create_Cycle $win \$size \[iota 1 \[expr (\$size-1)\]\]"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.size.label .params.size.field -side left
    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.actions -side top

    bind .params <Escape> {.params.actions.cancel invoke}

}

proc Create_Random { win size proba } {
    global mark
    global PI curzoomvalue object_distance place_radius create_props
    global DEBUG

    if { $size < 1 } { return }

    randomInit 1

    if $DEBUG {
	puts -nonewline "Creation of a random event graph with $size nodes"
    }

    set scale [expr $curzoomvalue/100.0]
    set OD [expr $object_distance*$scale]

    set Alpha [expr 2*$PI / $size];
    if [ expr { sin($Alpha/2.0) == 0.0 } ] {
	set Radius 0.0
    } else {
	set Radius [expr $OD / sin( $Alpha/2 )]
    }

    # putting the center... at the center
    set val $create_props(xoffset)
    set xorig [expr [winfo pixels . ${val}m]+$OD+$Radius]
    set val $create_props(yoffset)
    set yorig [expr [winfo pixels . ${val}m]+$OD+$Radius]

    # perhaps it is necessary to resize the canvas?
    Resize_Graph $win [expr $xorig+$Radius+$OD] \
	[expr $yorig+$Radius+$OD]

    # put transitions on a circle
    for { set i 0 } { $i < $size } { incr i 1 } {
	set xpos [xcoord $xorig $Radius [expr $i*$Alpha]]
	set ypos [ycoord $yorig $Radius [expr $i*$Alpha]]

	set tr_id($i) [Create_Trans_With_Props $win $xpos $ypos]

	if { "yes" == $create_props(is_recycled) } {
	    Create_Recycling $win $tr_id($i)
	}
    }

    # put places randomly...
    for { set i 0 } { $i < $size } { incr i 1 } {
	set xpos1 [xcoord $xorig $Radius [expr $i*$Alpha]]
	set ypos1 [ycoord $yorig $Radius [expr $i*$Alpha]]

	for { set j 0 } { $j < $size } { incr j 1 } {
	    if [expr { $i != $j } && { [random] < $proba }] {
		
		set xpos2 [xcoord $xorig $Radius [expr $j*$Alpha]]
		set ypos2 [ycoord $yorig $Radius [expr $j*$Alpha]]

		# put a place in between
		set xpos [expr ($xpos1+$xpos2)/2]
		set ypos [expr ($ypos1+$ypos2)/2]

		set pl_id [Create_Place_With_Props $win $xpos $ypos]

		# draw links 
		set li1_id [Put_TraPla_Link $win $tr_id($i) $pl_id {}]
		set li2_id [Put_PlaTra_Link $win $pl_id $tr_id($j) {}]
		if [info exists create_props(link_weight)] {
		    set link_weight($li1_id) $create_props(link_weight)
		    set link_weight($li2_id) $create_props(link_weight)
		}

		# (re)write text. Necessary because information may have 
		# changed
		Display_Weight $win $li1_id
		Display_Weight $win $li2_id
	    }
	}
    }
    if $DEBUG {
	puts " ...Done"
    }
}

proc Make_Random { win } {
    global size
    global DEBUG

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Number of nodes:" -width 20 -anchor w
    entry .params.size.field -textvar size -width 3 -relief sunken
    frame .params.proba
    label .params.proba.label -text "Edge probability:" -width 20 -anchor w
    entry .params.proba.field -textvar proba -width 12 -relief sunken

    pack .params.size.label .params.size.field -side left
    pack .params.proba.label .params.proba.field -side left
    pack .params.size .params.proba -side top -fill x

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n SetCreateLaws\n Create_Random $win \$size \$proba"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.actions -side top

    bind .params <Escape> {.params.actions.cancel invoke}

}

proc Create_String { win size orient } {
    global mark link_weight
    global PI curzoomvalue object_distance place_radius create_props
    global DEBUG

    set scale [expr $curzoomvalue/100.0]
    set OD [expr $object_distance*$scale]

    # defining the origin of the bounding rectangle
    set val $create_props(xoffset)
    set xorig [expr [winfo pixels . ${val}m]+$OD]
    set val $create_props(yoffset)
    set yorig [expr [winfo pixels . ${val}m]+$OD]

    # a first loop for computing the position of each node
    for { set i 0 } { $i < $size } { incr i 1 } {

        case $orient {
            {hor} {
                set X($i) [expr $xorig + 2*$OD*$i]
                set Y($i) $yorig
	    }
            {ver} {
		set X($i) $xorig
		set Y($i) [expr $yorig + 2*$OD*$i]
	    }
            {dia} {
		set X($i) [expr $xorig + 2*$OD*$i]
		set Y($i) [expr $yorig + 2*$OD*$i]
	    }
	    {cir} {
		# I know: this should not be recomputed each time.
		set Alpha [expr 2*$PI / $size];
		if [ expr { sin($Alpha/2.0) == 0.0 } ] {
		    set Radius 0.0
		} else {
		    set Radius [expr $OD / sin( $Alpha/2 )]
		}
		set xorig [expr int(1.2*$Radius)]
		set yorig [expr int(1.2*$Radius)]
		
		set X($i) [xcoord $xorig $Radius [expr $i*$Alpha]]
		set Y($i) [ycoord $yorig $Radius [expr $i*$Alpha]]
	    }
	    {rec} {
		set xnb [expr int(sqrt($size))]
		set row [expr $i / $xnb]
		set col [expr $i % $xnb]
		set X($i) [expr $xorig + 2*$OD*$row]
		set Y($i) [expr $yorig + 2*$OD*$col]
	    }
	}
    }

    # find out if resizing is necessary
    set xmax [TableMax X 0 [expr $size-1]]
    set ymax [TableMax Y 0 [expr $size-1]]
    Resize_Graph $win [expr $xmax+$OD] [expr $ymax+$OD]

    # now put the objects...
    for { set i 0 } { $i < $size } { incr i } {

	set xpos $X($i)
	set ypos $Y($i)

	set tr_id($i) [Create_Trans_With_Props $win $xpos $ypos]

	if { "yes" == $create_props(is_recycled) } {
	    Create_Recycling $win $tr_id($i)
	}
    }

    for { set i 0 } { $i < [expr $size-1] } { incr i } {
	set xpos1 $X($i)
	set ypos1 $Y($i)

	set j [expr $i+1]
	set xpos2 $X($j)
	set ypos2 $Y($j)

	set xpos [expr ($xpos1+$xpos2)/2]
	set ypos [expr ($ypos1+$ypos2)/2]

	set pl_id [Create_Place_With_Props $win $xpos $ypos]

	# draw links 
	set li1_id [Put_TraPla_Link $win $tr_id($i) $pl_id {}]
	set li2_id [Put_PlaTra_Link $win $pl_id $tr_id($j) {}]
	if [info exists create_props(link_weight)] {
	    set link_weight($li1_id) $create_props(link_weight)
	    set link_weight($li2_id) $create_props(link_weight)
	}

	# (re)write text. Necessary because information may have 
	# changed
	Display_Weight $win $li1_id
	Display_Weight $win $li2_id

    }
    if $DEBUG {
	puts " ...Done"
    }
}

proc Make_String { win } {

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Number of nodes:" -width 20 -anchor w
    entry .params.size.field -textvar size -width 3 -relief sunken

    pack .params.size.label .params.size.field -side left

    frame .params.orient
    label .params.orient.label -text "Orientation:" -width 20  -anchor w
    radiobutton .params.orient.hor -text "Horizontal" -variable orient \
	-value hor
    radiobutton .params.orient.ver -text "Vertical" -variable orient \
	-value ver
    radiobutton .params.orient.dia -text "Diagonal" -variable orient \
	-value dia
    radiobutton .params.orient.cir -text "Circular" -variable orient \
	-value cir
    radiobutton .params.orient.rec -text "Rectangular" -variable orient \
	-value rec
    pack .params.orient.label .params.orient.hor .params.orient.ver \
	.params.orient.dia .params.orient.cir .params.orient.rec \
	-side left -fill x
    pack .params.size .params.orient -side top -fill x

    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n SetCreateLaws\n Create_String $win \$size \$orient"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.actions -side top

    bind .params <Escape> {.params.actions.cancel invoke}

}

proc Create_JobShop { win sizex sizey } {
    global mark link_weight
    global PI curzoomvalue object_distance place_radius create_props
    global DEBUG

    if [expr { $sizex <= 0 } || { $sizey <= 0 }] { return }

    set scale [expr $curzoomvalue/100.0]
    set OD [expr $object_distance*$scale]

    # defining the origin of the bounding rectangle
    set val $create_props(xoffset)
    set xorig [expr [winfo pixels . ${val}m]+$OD]
    set val $create_props(yoffset)
    set yorig [expr [winfo pixels . ${val}m]+$OD]

    # find out if resizing is necessary
    set xmax [expr $xorig + $OD*(2*$sizex+1)]
    set ymax [expr $yorig + $OD*(2*$sizey+1)]
    Resize_Graph $win $xmax $ymax 

    for { set i 0 } { $i < $sizex } { incr i 1 } {
	for { set j 0 } { $j < $sizey } { incr j 1 } {

	    set num [expr $i*$sizey + $j]

	    set xpos [expr $xorig + 2*$OD*$i]
	    set ypos [expr $yorig + 2*$OD*$j]

	    # create the transitions
	    set tr_id($num) [Create_Trans_With_Props $win $xpos $ypos]

	    if { "yes" == $create_props(is_recycled) } {
		Create_Recycling $win $tr_id($num)
	    }

	    # create horizontal place
	    set xpos1 [expr $xpos + $OD]
	    set ypos1 [expr $ypos]
	    set pl_id [Create_Place_With_Props $win $xpos1 $ypos1]
	    set h_pl_id($num) $pl_id
	    # draw link
	    set li1_id [Put_TraPla_Link $win $tr_id($num) $pl_id {}]

	    # create vertical one
	    set xpos1 [expr $xpos]
	    set ypos1 [expr $ypos + $OD]
	    set pl_id [Create_Place_With_Props $win $xpos1 $ypos1]
	    set v_pl_id($num) $pl_id
	    # draw link
	    set li2_id [Put_TraPla_Link $win $tr_id($num) $pl_id {}]

	    # position link attributes
	    if [info exists create_props(link_weight)] {
		set link_weight($li1_id) $create_props(link_weight)
		set link_weight($li2_id) $create_props(link_weight)
	    }

	    # (re)write text. Necessary because information may have 
	    # changed
	    Display_Weight $win $li1_id
	    Display_Weight $win $li2_id

	}
    }

    # Now create links with previous place. This needs all place
    # ids. This is why there is a second loop.
    for { set i 0 } { $i < $sizex } { incr i 1 } {
	for { set j 0 } { $j < $sizey } { incr j 1 } {

	    set num [expr $i*$sizey + $j]

	    set xpos [expr $xorig + 2*$OD*$i]
	    set ypos [expr $yorig + 2*$OD*$j]

	    if { $i > 0 } {
		set pl_id $h_pl_id([expr $num-$sizey])
		set li1_id [Put_PlaTra_Link $win $pl_id $tr_id($num) {}]
	    } else {
		# more complicated return arc...
		set xpos1 [expr $xorig + (2*$sizex-1)*$OD]
		set ypos1 [expr $ypos]
		set xpos2 $xpos1
		set ypos2 [expr $ypos1 - $OD/2.5]
		set xpos3 [expr $xorig - $OD/3]
		set ypos3 $ypos2
		set xpos4 $xpos3
		set ypos4 $ypos

		set pl_id $h_pl_id([expr ($sizex-1)*$sizey+$j])
		set pts [list $xpos2 $ypos2 $xpos3 $ypos3 $xpos4 $ypos4 ]
		set li1_id [Put_PlaTra_Link $win $pl_id $tr_id($num) $pts]
	    }
	    if { $j > 0 } {
		set pl_id $v_pl_id([expr $num-1])
		set li2_id [Put_PlaTra_Link $win $pl_id $tr_id($num) {}]
	    } else {
		set xpos1 [expr $xpos]
		set ypos1 [expr $yorig + (2*$sizey-1)*$OD]
		set ypos2 $ypos1
		set xpos2 [expr $xpos1 - $OD/2.5]
		set ypos3 [expr $yorig - $OD/3]
		set xpos3 $xpos2
		set ypos4 $ypos3
		set xpos4 $xpos

		set pl_id $v_pl_id([expr $i*$sizey+$sizey-1])
		set pts [list $xpos2 $ypos2 $xpos3 $ypos3 $xpos4 $ypos4 ]
		set li2_id [Put_PlaTra_Link $win $pl_id $tr_id($num) $pts]

		# position link attributes
		if [info exists create_props(link_weight)] {
		    set link_weight($li1_id) $create_props(link_weight)
		    set link_weight($li2_id) $create_props(link_weight)
		}
	    }

	    # (re)write text. Necessary because information may have 
	    # changed
	    Display_Weight $win $li1_id
	    Display_Weight $win $li2_id

	}
    }
    if $DEBUG {
	puts " ...Done"    
    }
}

proc Make_JobShop { win } {
    global sizex sizey 

    if [catch {toplevel .params}] { return }

    frame .params.size
    label .params.size.label -text "Dimensions:" -width 20 -anchor w
    label .params.size.labelx -text "X=" -width 4 -anchor w
    entry .params.size.fieldx -textvar sizex -width 3 -relief sunken
    label .params.size.labely -text "Y=" -width 4 -anchor w
    entry .params.size.fieldy -textvar sizey -width 3 -relief sunken
    pack .params.size.label .params.size.labelx .params.size.fieldx \
	.params.size.labely .params.size.fieldy -side left
    pack .params.size -side top -fill x
    
    Control_Panel .params

    frame .params.actions
    button .params.actions.ok -text "Ok" \
	-command "destroy .params\n SetCreateLaws\n Create_JobShop $win \$sizex \$sizey"
    button .params.actions.cancel -text "Cancel" \
	-command "destroy .params"

    pack .params.actions.ok .params.actions.cancel -side left
    pack .params.actions -side top

    bind .params <Escape> {.params.actions.cancel invoke}

}

######################################################################
# Process_Maxplus_Matrix -
# A procedure to put (max,+) matrices in "canonical" format (i.e.
# the size followed by size*size entries).
# In particular, recognizes Scilab (max,+) matrices and cleans up
# the entries.
#
# Parameters:
# - matrix	a character string/block containing the matrix
#
proc Process_Maxplus_Matrix { matrix } {
    global DEBUG

    set res matrix

    if [regexp {\#\(} $matrix] {
	# this could be a Scilab matrix!! Extract it...
	regsub -all ";" $matrix {} matrix
	regsub -all "\n" $matrix {} matrix
	regsub -all "%0" $matrix "eps" matrix
	# getting rid of everything before the first "(." and after the
	# first ".)" (anything more intelligent has failed 8-( )
	regsub -all {[^\)]*\(.} $matrix {} matrix
	regsub -all {.\).*} $matrix {} matrix
	set num [llength $matrix]
	set size [expr int(sqrt($num))]
	if [expr $size*\$size != $num ] {
	    puts "Warning: bad number of entries found in matrix. Proceeding anyway..."
	}
	set matrix [linsert $matrix 0 $size]
    }

    if $DEBUG {
	puts "Clean=$matrix"
    }

    return $matrix
}

proc Create_From_Matrix { win file_name { merge 0 } } {
    global PlaceTime mark link_weight
    global PI curzoomvalue object_distance place_radius create_props
    global DEBUG

    set scale [expr $curzoomvalue/100.0]
    set OD [expr $object_distance*$scale]

    if ![file exists $file_name] {
	ReportError "Create Error" "Can't open file $file_name."
	return
    }
    # get matrix
    set matfile [open $file_name r]
    set matrix [read $matfile]
    set matrix [Process_Maxplus_Matrix $matrix]
    set size [lindex $matrix 0]

    if { $size < 1 } { return }

    if $DEBUG {
	puts -nonewline "Creation of an event graph from matrix file $file_name"
    }

    set Alpha [expr 2*$PI / $size];
    if [ expr { sin($Alpha/2.0) == 0.0 } ] {
	set Radius 0.0
    } else {
	set Radius [expr $OD / sin( $Alpha/2 )]
    }

    # putting the center... at the center
    set val $create_props(xoffset)
    set xorig [expr [winfo pixels . ${val}m]+$OD+$Radius]
    set val $create_props(yoffset)
    set yorig [expr [winfo pixels . ${val}m]+$OD+$Radius]

    # perhaps it is necessary to resize the canvas?
    Resize_Graph $win [expr $xorig+$Radius+$OD] [expr $yorig+$Radius+$OD]

    # put transitions on a circle
    for { set i 0 } { $i < $size } { incr i 1 } {
	set xpos [xcoord $xorig $Radius [expr $i*$Alpha]]
	set ypos [ycoord $yorig $Radius [expr $i*$Alpha]]

	set tr_id($i) [Make_Trans $win \
			   [list \
				-x $xpos \
				-y $ypos \
				-lawparams "name constant value 0.0" \
			       ] ]
    }

    # put places according to the matrix...
    for { set i 0 } { $i < $size } { incr i 1 } {
	set xpos1 [xcoord $xorig $Radius [expr $i*$Alpha]]
	set ypos1 [ycoord $yorig $Radius [expr $i*$Alpha]]

	for { set j 0 } { $j < $size } { incr j 1 } {
	    set val [lindex $matrix [expr $i*$size+$j+1]]
	    if { $val != "eps" } {
		
		if { $i == $j } {
		    set pl_id [Create_Recycling $win $tr_id($i)]
		    array set PlaceTime($pl_id) [list name constant value $val]
		    if $DEBUG {
			puts "Time of place $pl_id set to $val"
		    }
		} else {

		    set xpos2 [xcoord $xorig $Radius [expr $j*$Alpha]]
		    set ypos2 [ycoord $yorig $Radius [expr $j*$Alpha]]

		    # put a place in between
		    set xpos [expr ($xpos1+$xpos2)/2]
		    set ypos [expr ($ypos1+$ypos2)/2]
		    set pl_id [Make_Place $win \
				   [list \
					-x $xpos \
					-y $ypos \
					-marking 1 \
					-lawparams "name constant value $val" \
				       ] ]

		    # draw links 
		    set li1_id [Put_TraPla_Link $win $tr_id($i) $pl_id {}]
		    set li2_id [Put_PlaTra_Link $win $pl_id $tr_id($j) {}]
		    if [info exists create_props(link_weight)] {
			set link_weight($li1_id) $create_props(link_weight)
			set link_weight($li2_id) $create_props(link_weight)
		    }

		    # (re)write text. Necessary because information may have 
		    # changed
		    Display_Weight $win $li1_id
		    Display_Weight $win $li2_id

		}
	    }
	}
    }
    if $DEBUG {
	puts " ...Done"
    }
}

######################################################################
# Make_From_Matrix -
# Creates a user dialog interface for choosing a (max,+) matrix 
# file and setting up the parameters before the creation of the representation.
# At the moment, the geometry is fixed to a circle, but this may change
# in the future!
#
# Parameters:
# - canvas	the canvas window in which to create the objects
# 
proc Make_From_Matrix { win } {
    global size

    set types {
	{{MaxPlus matrix}       {.mat}  }
	{{All Files}        *           }
    }

    set options "-defaultextension mat -filetypes {$types} -parent ."

    set mat_file_name [GetOpenFile $options]

    if { $mat_file_name != "" } {
	Create_From_Matrix $win $mat_file_name
    }
}
